@@ -16,7 +16,7 @@
 #' @method predict cdfqr 
 #' @export 
 #' 
-predict.cdfqr <- function(object, newdata = NULL, type = c("full","mu","sigma"), quant = 0.5,...) {
+predict.cdfqr <- function(object, newdata = NULL, type = c("full","mu","sigma", "theta", "one", "zero"), quant = 0.5,...) {
   type <- match.arg(type)
   
  if (!is.null(newdata)){
@@ -27,17 +27,23 @@ predict.cdfqr <- function(object, newdata = NULL, type = c("full","mu","sigma"),
    m <- model.frame(Terms, newdata)
    X <- model.matrix(Terms, m)
    
-   
-   betas.lm <- object$coefficients$location[,"Estimate"]
-   betas.pm <- object$coefficients$dispersion[,"Estimate"]
-   term.lm <- rownames(object$coefficients$location)
-   term.pm <- rownames(object$coefficients$dispersion)
-   
-   if(length(term.lm) < 2){
-     predictlm <- X[, term.lm] * betas.lm
+
+   if(!is.null(object$coefficients$location)){
+     betas.lm <- object$coefficients$location[,"Estimate"]
+     term.lm <- rownames(object$coefficients$location)
+     
+     if(length(term.lm) < 2){
+       predictlm <- X[, term.lm] * betas.lm
+     }else{
+       predictlm <- X[, term.lm] %*% betas.lm
+     }
    }else{
-     predictlm <- X[, term.lm] %*% betas.lm
+     predictlm = NULL
    }
+
+   
+   betas.pm <- object$coefficients$dispersion[,"Estimate"]
+   term.pm <- rownames(object$coefficients$dispersion)
    
    if(length(term.pm) < 2){
      predictpm <- X[, term.pm] * betas.pm
@@ -47,38 +53,91 @@ predict.cdfqr <- function(object, newdata = NULL, type = c("full","mu","sigma"),
   
    predictsigma <- exp(predictpm)
    
+   
+   if("cdfqrFT"%in%class(object)){
+     
+     betas.tm <- object$coefficients$skew[,"Estimate"]
+     term.tm <- rownames(object$coefficients$skew)
+     
+     if(length(term.tm) < 2){
+       predicttm <- X[, term.tm] * betas.tm
+     }else{
+       predicttm <- X[, term.tm] %*% betas.tm
+     }
+     
+     predicttheta <- predicttm
+     
+   }
+   
    n <- length(predictsigma)
    if (length(quant) == 1) {
      q_y <- rep(quant, n)
    }else if (length(quant) == n){
      q_y <- quant
    }
-  
-   pred <- qq(q_y, predictlm, predictsigma, fd, sd)
+   
+   if("cdfqrFT"%in%class(object)){
+     modelspecific <- object$distinf
+     pred <- qqft(q_y, predictsigma, predicttheta, fd, sd, 
+                  mu = predictlm, inner = object$distinf$inner, 
+                  version = object$distinf$version)
+     
+   }else{
+     pred <- qq(q_y, predictlm, predictsigma, fd, sd)
+   }
+
+   
+   
+   if("cdfqrH"%in%class(object)){
+     if (!is.null(object$mod_zp)){
+       pred_zero <- predict(object$mod_zp, newdata = newdata,
+                            type = "response")
+     }
+     if (!is.null(object$mod_op)){
+       pred_one <- predict(object$mod_op, newdata = newdata,
+                           type = "response")
+     }
+     
+   }
+
+   
+   
+   
    fitted <- switch(type, full = {
      pred
    }, mu = {
      predictlm
    }, sigma = {
      predictsigma
+   }, theta = {
+     predicttheta
+   }, zero = {
+     pred_zero
+   }, one = {
+     pred_one
    })
  }else{
    fitted <- switch(type, full = {
      object$fitted$full
    }, mu = {
-     object$fitted$full_mu
+     object$fitted$mu
    }, sigma = {
-     object$fitted$full_sigma
+     object$fitted$sigma
+   }, theta = {
+     object$fitted$theta
+   }, zero = {
+     object$fitted$zerop
+   }, one = {
+     object$fitted$onep
    })
  }
   
   return(fitted)
 }
 
-#' @method fitted cdfqr 
 #' @export 
 #' @rdname predict.cdfqr
-fitted.cdfqr <- function(object, type = c("full","mu","sigma"), plot = FALSE, ...) {
+fitted.cdfqr <- function(object, type = c("full","mu","sigma", "theta", "one", "zero"), plot = FALSE, ...) {
   type <- match.arg(type)
   
   fitted <- switch(type, full = {
@@ -87,117 +146,25 @@ fitted.cdfqr <- function(object, type = c("full","mu","sigma"), plot = FALSE, ..
     object$fitted$mu
   }, sigma = {
     object$fitted$sigma
-  })
-  if (plot) {
-    plot1 <- plot(object)
-    return(list(fitted = fitted,
-           plot = plot1))
-  }else{
-    return(fitted)
-    }
-
-}
-
-
-#' @method predict cdfqrH 
-#' @export 
-#' @rdname predict.cdfqr
-predict.cdfqrH <- function(object, newdata = NULL, 
-                           type = c("full","mu","sigma","zero","one"), 
-                           quant = 0.5,...) {
-  type <- match.arg(type)
-  
-  if (!is.null(newdata)){
-    tt <- terms(object$formula)
-    Terms <- delete.response(tt)
-    fd <- object$family$fd
-    sd <- object$family$sd
-    m <- model.frame(Terms, newdata)
-    X <- model.matrix(Terms, m)
-    
-    
-    betas.lm <- object$coefficients$location[,"Estimate"]
-    betas.pm <- object$coefficients$dispersion[,"Estimate"]
-    term.lm <- rownames(object$coefficients$location)
-    term.pm <- rownames(object$coefficients$dispersion)
-    
-    if(length(term.lm) < 2){
-      predictlm <- X[, term.lm] * betas.lm
-    }else{
-      predictlm <- X[, term.lm] %*% betas.lm
-    }
-    
-    if(length(term.pm) < 2){
-      predictpm <- X[, term.pm] * betas.pm
-    }else{
-      predictpm <- X[, term.pm] %*% betas.pm
-    }
-    
-    predictsigma <- exp(predictpm)
-    
-    n <- length(predictsigma)
-    if (length(quant) == 1) {
-      q_y <- rep(quant, n)
-    }else if (length(quant) == n){
-      q_y <- quant
-    }
-    
-    pred <- qq(q_y, predictlm, predictsigma, fd, sd)
-    
-    if (!is.null(object$mod_zp)){
-      pred_zero <- predict(object$mod_zp, newdata = newdata,
-                           type = "response")
-    }
-    if (!is.null(object$mod_op)){
-      pred_one <- predict(object$mod_op, newdata = newdata,
-                           type = "response")
-    }
-  
-    fitted <- switch(type, full = {
-      pred
-    }, mu = {
-      predictlm
-    }, sigma = {
-      predictsigma
-    }, zero = {
-      pred_zero
-    }, one = {
-      pred_one
-    })
-  }else{
-    fitted <- switch(type, full = {
-      object$fitted$full
-    }, mu = {
-      object$fitted$full_mu
-    }, sigma = {
-      object$fitted$full_sigma
-    }, zero = {
-      object$fitted$zerop
-    }, one = {
-      object$fitted$onep
-    })
-  }
-  
-  return(fitted)
-}
-
-#' @method fitted cdfqrH 
-#' @export 
-#' @rdname predict.cdfqr
-fitted.cdfqrH <- function(object,  type = c("full","mu","sigma","zero","one"), ...) {
-  type <- match.arg(type)
-  fitted <- switch(type, full = {
-    object$fitted$full
-  }, mu = {
-    object$fitted$full_mu
-  }, sigma = {
-    object$fitted$full_sigma
+  }, theta = {
+    object$fitted$theta
   }, zero = {
     object$fitted$zerop
   }, one = {
     object$fitted$onep
   })
   
-  return(fitted)
-  
-}
\ No newline at end of file
+  if (plot) {
+    if("cdfqrH"%in%class(object)){
+      cat("Plot is not available for hurdle model and finite tailed model currently")
+    }else{
+      plot1 <- plot(object)
+      return(list(fitted = fitted,
+                  plot = plot1))
+    }
+
+  }else{
+    return(fitted)
+    }
+
+}
